package imp.ringing;

import java.io.UnsupportedEncodingException;

/**
 * Encodes bits in blocks of 6 in the following way.
 * 	Whitespace - 0
 * 	Uppercase letters - 1..26
 *  Lowercase letters - 27..52
 *  Numbers - 53..62
 *  Period - 63
 */
public class AlphaNumCoder extends Coder {

	@Override
	public boolean[] encode(String s) throws UnsupportedEncodingException {
		boolean[] ret = new boolean[6 * s.length()];
				
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			int x = 0;
			
			if (Character.isWhitespace(c)) {
				x = 0;
			} else if (c >= 'A' && c <= 'Z') {
				x = Character.getNumericValue(c) - Character.getNumericValue('A') + 1;
			} else if (c >= 'a' && c <= 'z') {
				x = Character.getNumericValue(c) - Character.getNumericValue('a') + 27;
			} else if (c >= '0' && c <= '9') {
				x = Character.getNumericValue(c) - Character.getNumericValue('0') + 53;
			} else if (c == '.') {
				x = 63;
			} else {
				throw new UnsupportedEncodingException(
						"Only letters, numbers, space and dot can be used.");
			}
			
			String t = Integer.toBinaryString(x);
			for (int j = 0; j < 6; j++) {
				if (j < t.length())
					ret[i * 6 + j] = (t.charAt(t.length() - 1 - j) == '1');
				else
					ret[i * 6 + j] = false;				
			}
		}
				
		return ret;
	}

	@Override
	protected String decode(Boolean[] s) {
		StringBuilder ret = new StringBuilder();
		
		for (int i = 0; i < s.length / 6; i++) {
			int value = 0;
			char c;
			
			for (int j = 0; j < 6; j++)
				if (s[i * 6 + j])
					value |= 1 << j;
			
			if (value == 0)
				c = ' ';
			else if (value < 27)
				c = (char)((int)'A' + value - 1);
			else if (value < 53)
				c = (char)((int)'a' + value - 27);
			else if (value < 63)
				c = (char)((int)'0' + value - 53);
			else
				c = '.';
			
			ret.append(c);
		}
		
		return ret.toString();
	}

}
