package imp.ringing;

import android.util.Log;
import android.widget.EditText;

/**
 * A special protocol used to measure event timings between the participants. 
 */
public class DebugProtocol extends Protocol {
	private EditText target = (EditText)Main.context.findViewById(R.id.memoSentMsg);
	private long timer;
	
	/**
	 * Initializes the Debug protocol.
	 * @param params Parameters.
	 */
	public DebugProtocol(Parametrization params) {
		super(params);
		setProtocolState(ProtocolState.Inactive);
	}

	/**
	 * @return Current time (rounded to 100s).
	 */
	private String getCurrentTime() {
		return "(" + (System.currentTimeMillis() % 100000) + ") ";
	}
	
	/**
	 * Output measurements directly to the UI, both for the sender and receiver.
	 * Note that normally protocols' output is managed through the Receiver class,
	 * but this is not applicable here.
	 * @param text Text to output.
	 */
	private void output(final String text) {
		Main.context.runOnUiThread(
			new Runnable() {
				public void run() {
					target.append(text);
				}
			}
		);
	}
	
	/**
	 * Sets protocol state and output target, so that we output to the sender's
	 * edit in Sending state and to the receiver's edit in the Receiving state.
	 */
	@Override
	public void setProtocolState(ProtocolState state) {
		switch (state) {
			case Sending:
				target = (EditText)Main.context.findViewById(R.id.memoSentMsg);
				Main.context.runOnUiThread(
					new Runnable() {
						public void run() {
							target.setText("");
						}
					}
				);
				Log.i("USER", "Sender: Debug to memoSent");
			break;
			
			case Receiving:
				target = (EditText)Main.context.findViewById(R.id.memoRecvMsg);
				Log.i("USER", "Sender: Debug to memoRecv");
			break;
			
			case Inactive:
				target = new EditText(Main.context);
				Log.i("USER", "Sender: No debug");
			break;
		}
		
		super.setProtocolState(state);
	}
	
	@Override
	public void send(boolean b, boolean last) throws InterruptedException {
		output("CS: " + getCurrentTime() + "\r\n");
		timer = getElapsedTimeMillis();

		ring();
		this.mutexUp();
		
		if (!last) {
			Thread.sleep(10000);
		}
	}
	
	@Override
	public void callingCallback() {
		output("CO: " + getCurrentTime() + (getElapsedTimeMillis() - timer) + "\r\n");
		timer = getElapsedTimeMillis();
	};
	
	@Override
	public void callingHangupCallback() {
		output("CH: " + getCurrentTime() + (getElapsedTimeMillis() - timer) + "\r\n");
		hangup();
		this.mutexDown();
	}
	
	@Override
	public void receivingCallback(String number) {
		output("RC: " + getCurrentTime() + "\r\n");
		timer = getElapsedTimeMillis();
		
		try {
			Thread.sleep(5000);
		} catch (Exception e) {
		}

		output("RX: " + getCurrentTime() + (getElapsedTimeMillis() - timer) + "\r\n");
		timer = getElapsedTimeMillis();
		hangup();
	};

	@Override
	public void receivingHangupCallback() {
		output("RH: " + getCurrentTime() + (getElapsedTimeMillis() - timer) + "\r\n");
		
		this.mutexDown();
	};
	
	@Override
	public boolean receive() throws InterruptedException {
		this.mutexUp();
		return false;
	}

}
