package imp.ringing;

import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioGroup;


/**
 * A static class that handles UI events. 
 */
public class Handlers {

	/** The application mode radio handler. */
	public static final RadioGroup.OnCheckedChangeListener modeRadioHandler = new RadioGroup.OnCheckedChangeListener() {
		@Override
		public void onCheckedChanged(RadioGroup group, int checkedId) {
			if (checkedId == R.id.radioModeSend) {
				Main.context.findViewById(R.id.panelSend).setVisibility(View.VISIBLE);
				Main.context.findViewById(R.id.panelRecv).setVisibility(View.GONE);
				
				Receiver.stop();
			} else {
				Main.context.findViewById(R.id.panelRecv).setVisibility(View.VISIBLE);
				Main.context.findViewById(R.id.panelSend).setVisibility(View.GONE);
				
				startReceiver();
			}
		}
	};

	/** The protocol radio handler. */
	public static final RadioGroup.OnCheckedChangeListener protocolRadioHandler = new RadioGroup.OnCheckedChangeListener() {
		@Override
		public void onCheckedChanged(RadioGroup group, int checkedId) {
			ProtocolSelector.selectByIndex(group.indexOfChild(Main.context.findViewById(checkedId)));			
			
			if (Receiver.isActive()) {
				Receiver.stop();
				startReceiver();
			}
		}
	};

	/** The encoding radio handler. */
	public static final RadioGroup.OnCheckedChangeListener encodingRadioHandler = new RadioGroup.OnCheckedChangeListener() {
		@Override
		public void onCheckedChanged(RadioGroup group, int checkedId) {
			CoderSelector.selectByIndex(group.indexOfChild(Main.context.findViewById(checkedId)));
			
			if (Receiver.isActive()) {
				Receiver.stop();
				startReceiver();
			}			
		}
	};
	
	/** Clears the memo in which the received message is output. */
	private static void clearReceivingMemo() {
		((EditText)Main.context.findViewById(R.id.memoRecvMsg)).setText("");
	}
	
	/** @return The parameters entered by user as a Parametrization object. */
	private static Parametrization getParams() {
		Parametrization params = new Parametrization();
		
		params.setD(
			Double.parseDouble(((EditText)Main.context.findViewById(R.id.editMinD)).getText().toString()),
			Double.parseDouble(((EditText)Main.context.findViewById(R.id.editMaxD)).getText().toString())
		);

		params.setH(
			Double.parseDouble(((EditText)Main.context.findViewById(R.id.editMinH)).getText().toString()),
			Double.parseDouble(((EditText)Main.context.findViewById(R.id.editMaxH)).getText().toString())
		);
		
		params.setRingLength(
			Double.parseDouble(((EditText)Main.context.findViewById(R.id.editRingTime)).getText().toString())
		);
		
		return params;
	}
	
	/** Starts the receiving process. */ 
	private static void startReceiver() {
		clearReceivingMemo();
		Receiver.setTarget((EditText)Main.context.findViewById(R.id.memoRecvMsg));
		Receiver.receive(ProtocolSelector.getProtocol(Handlers.getParams()), CoderSelector.getCoder());
	}
	
	/** Starts the sending process. */
	public static final Button.OnClickListener sendButtonHandler = new Button.OnClickListener() {
		@Override
		public void onClick(View view) {
			String message = ((EditText)(Main.context.findViewById(R.id.memoSentMsg))).getText().toString();
			String number = Handlers.getInputNumber();
			
			if (message.equals("")) {
				Dialogs.messageBox("Sorry", "The message must not be empty.");
				return;
			}
			
			if (number.equals("")) {
				Dialogs.messageBox("Sorry", "You have not entered the phone number.");
				return;
			}
			
			(new Thread(
				new Sender(
					ProtocolSelector.getProtocol(Handlers.getParams()),
					message,
					number,
					CoderSelector.getCoder()
				)
			)).start();
		}
						
	};
	
	/** Clears the receiving memo and restarts the Receiver. */
	public static final Button.OnClickListener clearRecvButtonHandler = new Button.OnClickListener() {
		@Override
		public void onClick(View view) {
			clearReceivingMemo();
			
			if (Receiver.isActive()) {
				Receiver.stop();
				startReceiver();
			}
		}
	};
	
	/** Clears the sending memo. */
	public static final Button.OnClickListener clearSentButtonHandler = new Button.OnClickListener() {
		@Override
		public void onClick(View view) {
			((EditText)Main.context.findViewById(R.id.memoSentMsg)).setText("");
		}
	};
		
	/** @return The phone number to call. */ 
	public static String getInputNumber() {
		return ((EditText)Main.context.findViewById(R.id.editNumber)).getText().toString().trim();
	}
}
