package imp.ringing;

/**
 * A storage class for the parameters. 
 */
public class Parametrization {
	private long dMin;
	private long dMax;
	private long hMin;
	private long hMax;
	private long ringLength;
	
	/**
	 * Sets the d parameter.
	 * @param min d_{min} in seconds.
	 * @param max d_{max} in seconds.
	 */
	public void setD(double min, double max) {
		dMin = (long)(min * 1000);
		dMax = (long)(max * 1000);
	}

	/**
	 * Sets the h parameter.
	 * @param min h_{min} in seconds.
	 * @param max h_{max} in seconds.
	 */	
	public void setH(double min, double max) {
		hMin = (long)(min * 1000);
		hMax = (long)(max * 1000);
	}
	
	/**
	 * Sets the l parameter.
	 * @param length The l parameter in seconds.
	 */
	public void setRingLength(double length) {
		ringLength = (long)(length * 1000);
	}
	
	/** @return d_{min} in ms. */
	public long getDMin() { return dMin; }
	/** @return d_{max} in ms. */
	public long getDMax() { return dMax; }
	/** @return h_{min} in ms. */	
	public long getHMin() { return hMin; }
	/** @return h_{max} in ms. */	
	public long getHMax() { return hMax; }
	/** @return l in ms. */
	public long getRingLength() { return ringLength; }
	/** @return Delta d in ms. */
	public long getDeltaD() { return dMax - dMin; }
	/** @return Delta h in ms. */
	public long getDeltaH() { return hMax - hMin; }
}
