package imp.ringing;

import android.util.Log;

/**
 * A static class used to select a subclass of Protocol. 
 */
public class ProtocolSelector {
	private static Class<? extends Protocol> selectedProtocol = null;
		
	/**
	 * Selects a protocol.
	 * @param index Index of the protocol in order in which they are listed in the UI.
	 */
	public static void selectByIndex(int index) {		
		switch (index) {
			case 0:
				Log.i("USER", "Selected OneTwoProtocol");
				selectedProtocol = OneTwoProtocol.class;
			break;
			
			case 1:
				Log.i("USER", "Selected LengthProtocol");
				selectedProtocol = LengthProtocol.class;
			break;
			
			case 2:
				Log.i("USER", "Selected RingSilenceProtocol");
				selectedProtocol = RingSilenceProtocol.class;
			break;
			
			case 3:
				Log.i("USER", "Selected AlternatingProtocol");
				selectedProtocol = AlternatingProtocol.class;
			break;
			
			case 4:
				Log.i("USER", "Selected DebugProtocol");
				selectedProtocol = DebugProtocol.class;
			break;
						
			default:
				Log.i("USER", "Unsupported protocol number");
		}
	}
	
	/**
	 * Returns the selected protocol.
	 * @param params Parameters to initialize the protocol with.
	 * @return The selected protocol initialized with given paramters.
	 */
	public static Protocol getProtocol(Parametrization params) {
		if (selectedProtocol == null) {
			selectByIndex(0);
		}
		
		try {
			return selectedProtocol.getConstructor(Parametrization.class).newInstance(params);
		} catch (Exception e) {
			// As selectedProtocol is never an abstract class or null,
			// this will never happen.
			return null;
		}
	}
}
