package imp.ringing;

import android.util.Log;
import android.widget.EditText;

/**
 * This class runs a Protocol in the receiving mode and outputs the received message. 
 */
public class Receiver {
	private static Protocol protocol;
	private static Coder coder;
	private static EditText target;
	private static Thread thread = null;
	private static boolean active = false;

	/**
	 * Sets this Receiver's output target to a given edit.
	 * @param target An edit into which the message is to be output. 
	 */
	public static void setTarget(EditText target) {
		Receiver.target = target;
	}
	
	/**
	 * Starts listening to incoming messages with a given protocol.
	 * @param protocol The protocol to use.
	 * @param coder The coder with which the message is to be decoded.
	 */
	public static void receive(Protocol protocol, Coder coder) {
		Receiver.protocol = protocol;
		Receiver.protocol.setProtocolState(ProtocolState.Receiving);
		Main.callBackProtocol = Receiver.protocol;
		
		Receiver.coder = coder;
		Receiver.coder.clear();
		
		Receiver.thread = new Thread(
			new Runnable() {
				@Override
				public void run() {
					Receiver.receivingLoop();
				}
			}
		);
		
		Receiver.thread.start();
		Log.i("USER", "Recvr : Receiving started");		
	}
	
	
	/**
	 * Stops listening to incoming messages.
	 */
	public static void stop() {
		Receiver.thread.interrupt();
		Receiver.thread = null;
	}
	
	/**
	 * Runs the receiving loop which calls the receive() function of the used protocol. 
	 */
	public static void receivingLoop() {
		try {
			active = true;
			
			while (true) {
				Log.i("USER", "Recvr : Listening");
				final boolean b = Receiver.protocol.receive();
				
				Log.i("USER", "Recvr : Outputting " + (b ? "1" : "0"));
				Main.context.runOnUiThread(
					new Runnable() {
						public void run() {
							Receiver.coder.append(b);
							Receiver.target.append(Receiver.coder.decodeNewData());
						}
					}
				);
			}
		} catch (InterruptedException e) {
			Log.i("USER", "Recvr : Receiving interrupted");
			active = false;
			return;
		}
	}
	
	
	/**
	 * @return True if the Receiver is currently listening.
	 */
	public static boolean isActive() {
		return active;
	}
	
}
